#include "pekon.h"

struct sheets *sctl;

void paletteInit(void)
{
	static unsigned char rgb[16 * 3] = {
		0x00, 0x00, 0x00,
		0xff, 0x00, 0x00,
		0x00, 0xff, 0x00,
		0xff, 0xff, 0x00,
		0x00, 0x00, 0xff,
		0xff, 0x00, 0xff,
		0x00, 0xff, 0xff,
		0xff, 0xff, 0xff,
		0xc6, 0xc6, 0xc6,
		0x84, 0x00, 0x00,
		0x00, 0x84, 0x00,
		0x84, 0x84, 0x00,
		0x00, 0x00, 0x84,
		0x84, 0x00, 0x84,
		0x00, 0x84, 0x84,
		0x84, 0x84, 0x84
	};
	paletteSet(rgb, 0, 15);
	return;
}

void paletteSet(unsigned char *rgb, int start, int end)
{
	int i, e;
	e = eflagsLoad();
	cli();
	out8(0x03c8, start);
	for (i = start; i <= end; i++) {
		out8(0x03c9, rgb[0] / 4);
		out8(0x03c9, rgb[1] / 4);
		out8(0x03c9, rgb[2] / 4);
		rgb += 3;
	}
	eflagsStore(e);
	return;
}

void shtInit(struct memory *mem)
{
	int i;
	sctl = (struct sheets *) memAlloc(mem, sizeof(struct sheets));
	sctl->top = -1;
	for (i = 0; i < SHEETS; i++) {
		sctl->sheet[i].flag = 0;
	}
	return;
}

struct sheet *shtAlloc(void)
{
	struct sheet *sht;
	int i;
	for (i = 0; i < SHEETS; i++) {
		if (sctl->sheet[i].flag == 0) {
			sht = &sctl->sheet[i];
			sht->flag = SHTUSE;
			sht->height = -1;
			return sht;
		}
	}
	return 0;
}

void shtSet(struct sheet *sht, unsigned char *buf, int sizex, int sizey, int cinv)
{
	sht->buf = buf;
	sht->sizex = sizex;
	sht->sizey = sizey;
	sht->cinv = cinv;
	return;
}

void shtUpdown(struct sheet *sht, int height)
{
	int h, old = sht->height;
	if (height > sctl->top + 1) {
		height = sctl->top + 1;
	} else if (height < -1) {
		height = -1;
	}
	sht->height = height;
	if (old > height) {
		if (height >= 0) {
			for (h = old; h > height; h--) {
				sctl->sht[h] = sctl->sht[h - 1];
				sctl->sht[h]->height = h;
			}
			sctl->sht[height] = sht;
		} else {
			if (sctl->top > old) {
				for (h = old; h < sctl->top; h++) {
					sctl->sht[h] = sctl->sht[h + 1];
					sctl->sht[h]->height = h;
				}
			}
			sctl->top--;
		}
		shtRefresh();
	} else if (old < height) {
		if (old >= 0) {
			for (h = old; h < height; h++) {
				sctl->sht[h] = sctl->sht[h + 1];
				sctl->sht[h]->height = h;
			}
			sctl->sht[height] = sht;
		} else {
			for (h = sctl->top; h >= height; h--) {
				sctl->sht[h + 1] = sctl->sht[h];
				sctl->sht[h + 1]->height = h + 1;
			}
			sctl->sht[height] = sht;
			sctl->top++;
		}
		shtRefresh();
	}
	return;
}

void shtRefresh(void)
{
	int h, bx, by, vx, vy;
	unsigned char *buf, c, *vram = (char *) 0xa0000;
	struct sheet *sht;
	for (h = 0; h <= sctl->top; h++) {
		sht = sctl->sht[h];
		buf = sht->buf;
		for (by = 0; by < sht->sizey; by++) {
			vy = sht->y + by;
			for (bx = 0; bx < sht->sizex; bx++) {
				vx = sht->x + bx;
				c = buf[by * sht->sizex + bx];
				if (c != sht->cinv) {
					vram[vy * 320 + vx] = c;
				}
			}
		}
	}
	return;
}

void shtSlide(struct sheet *sht, int x, int y)
{
	sht->x = x;
	sht->y = y;
	if (sht->height >= 0) {
		shtRefresh();
	}
	return;
}

void shtFree(struct sheet *sht)
{
	if (sht->height >= 0) {
		shtUpdown(sht, -1);
	}
	sht->flag = 0;
	return;
}

void boxfill(struct sheet *sht, int c, int x0, int y0, int x1, int y1)
{
	int x, y;
	for (y = y0; y <= y1; y++) {
		for (x = x0; x <= x1; x++) {
			sht->buf[y * sht->sizex + x] = c;
		}
	}
	return;
}

void putfont(struct sheet *sht, int c, int x, int y, char *font)
{
	int i;
	char *p, d;
	for (i = 0; i < 16; i++) {
		p = sht->buf + (y + i) * sht->sizex + x;
		d = font[i];
		if ((d & 0x80) != 0) { p[0] = c; }
		if ((d & 0x40) != 0) { p[1] = c; }
		if ((d & 0x20) != 0) { p[2] = c; }
		if ((d & 0x10) != 0) { p[3] = c; }
		if ((d & 0x08) != 0) { p[4] = c; }
		if ((d & 0x04) != 0) { p[5] = c; }
		if ((d & 0x02) != 0) { p[6] = c; }
		if ((d & 0x01) != 0) { p[7] = c; }
	}
	return;
}

void putfonts(struct sheet *sht, int c, int x, int y, unsigned char *s)
{
	extern char hankaku[4096];
	for (; *s != 0x00; s++) {
		putfont(sht, c, x, y, hankaku + *s * 16);
		x += 8;
	}
	return;
}

void boxframe(struct sheet *sht, int c, int x0, int y0, int x1, int y1)
{
	boxfill(sht, c, x0, y0, x1, y0);
	boxfill(sht, c, x0, y0, x0, y1);
	boxfill(sht, c, x0, y1, x1, y1);
	boxfill(sht, c, x1, y0, x1, y1);
	return;
}
